@@ -3,13 +3,14 @@ BEGIN {
   $Hailo::AUTHORITY = 'cpan:AVAR';
 }
 BEGIN {
-  $Hailo::VERSION = '0.64';
+  $Hailo::VERSION = '0.65';
 }
 
 use 5.010;
 use autodie qw(open close);
 use Any::Moose;
 use Any::Moose 'X::StrictConstructor';
+use File::Glob ':glob';
 use Class::Load qw(try_load_class);
 use Scalar::Util qw(blessed);
 use List::Util qw(first);
@@ -72,6 +73,14 @@ has brain_resource => (
     },
 );
 
+sub BUILD {
+    my ($self) = @_;
+    my $brain = $self->brain;
+    return if !defined $brain;
+    $self->brain(bsd_glob($brain));
+    return;
+}
+
 my %has = (
     engine => {
         name => 'Engine',
